/*  
    A class for menu buttons
    @Author Ethan White
    @class
*/
function Button()
{
	
	//Represents what type of button this is
	this.type = 0;
	/*
	 * Button types, by number:
	 * 
	 * 0 = Null button
	 * 1 = New Game
	 * 2 = Load Game
	 * 3 = Level Select
	 * 4 = Credits
	 * 5 = Instructions
	 * 
	 * Add more as needed
	 */
	
	//The image used for this button
	this.image = null;
	
	
	/**
	 * Initialises the object
	 * @param image     The image used for the button
	 * @param type      An integer representing what kind of button this is 
	 */
	this.startupButton = function(image, type, x, y)
	{
		this.startupVisualGameObject(image, x, y, 1);
		this.image = image;
		this.type = type;
	}
	
	
	/**
	 * Figures out whether the button has been clicked (by a mouse)
	 * @param event		The event associated with the click 
	 */
	this.onMouseDown = function(event)
	{
		//If the click is within the boundaries of this button
		if (g_MouseX >= this.x && g_MouseX <= (this.x + this.image.width)
			&& g_MouseY >= this.y && g_MouseY <= (this.y + this.image.height))
			{
				this.buttonClicked();
			}
	}
	
	//Same as above, but for touches
	this.touchStart = function(event)
	{
		//If the click is within the boundaries of this button
		if (g_MouseX >= this.x && g_MouseX <= (this.x + this.image.width)
			&& g_MouseY >= this.y && g_MouseY >= (this.y + this.image.height))
			{
				this.buttonClicked();
			}		
	}
	
	//Called when this button has been clicked
	this.buttonClicked = function()
	{
		//Determine what to do based on type
		switch (this.type)
		{
			
			case 0: //Null Button
				break;
				
				
			case 1: //New Game
				//play get item sound (start and end positions, can be found in SoundManager)
    			g_SoundManager.playSound(0.122, 0.610);
    			g_GameObjectManager.playAnim(g_ResourceManager.winPortalOut);
    			window.setTimeout(function(){g_ApplicationManager.hop(5);}, 300);
				//g_ApplicationManager.startLevel();
				break;
				
			case 2: //Load Game - ADD LATER
				break;
				
			case 3: //Level Select - ADD LATER
				break;
				
			case 4: //Credits - ADD LATER
				break;
				
			case 5: //Instructions
				g_ApplicationManager.openMainMenu(4)
				break;
				
			default: break;
		}		
	}
	
	//Shuts down this button
	this.shutdown = function()
	{
		g_GameObjectManager.removeGameObject(this);
	}
}

Button.prototype = new VisualGameObject;
