/*  
    A class to represent a past version of the player on the screen
    @Author Jonathan Saunders
    @class
*/
function CopyPlayer()
{    
    this.hasItem = false;   // True if the copy is holding the item, false otherwise
    
    this.touching = false;   //True if the item was just lost, and can't be regained during this collision with it
    
    this.animation = 4;     // Keeps track of which animation is currently being played
    /*
     * Animations reference:
     * 1 = Idle left
     * 2 = Idle right
     * 3 = Idle up
     * 4 = Idle down
     * 5 = Run left
     * 6 = Run right
     * 7 = Run up
     * 8 = Run down
     * More can be added as necessary
     */
    
    this.frames = 8;        //Number of frames in player animations
    
    this.dead = false;		// True if the copy should pop
    
    this.touching = false;     //True if the item was just lost, and can't be regained during this collision
    
    this.level = null;		// A reference to the level object  @type Level
    this.record = null;		// A reference to the past record object  @type PastRecord
    this.index;				// The index of this copy in the g_Copies array  @type Number
    
    this.ticks = 0;			// What tick the game is currently at
    
    // Initialises this object
    this.startupCopyPlayer = function(level, record, index)
    {
        this.startupAnimatedGameObject(g_ResourceManager.idleDownCopy, level.startX, level.startY, 5, this.frames, 10);
        this.level = level;
		this.record = record;
		this.ticks = 0;
		
		//Adjust the collision area
        this.collideY = 35 * g_RatioY;
        this.collideHeight = -40 * g_RatioY;
        this.collideX = 5 * g_RatioX;
        this.collideWidth = -10 * g_RatioX;
		this.dead = false;
        return this;
    }

    //Called when this copy reaches the level end post
    this.hitLevelEnd = function()
    {
    		this.hasItem = false;
    		
    		//Plays fail sound (start and end positions, can be found in SoundManager)
    		g_SoundManager.playSound(3.580, 4.615);
    		
			g_Records.pop();
			g_Copies.pop();
    		g_ApplicationManager.hop(3);
    	

    }


    /*  Updates the current animation depending on the movement
        of the player. */
    this.updateAnimation = function(index)
    {
    	
    	switch(index) {
    		
    		//Idle left
    		case 1:
    			if (this.animation != 1)
    			{
	    			this.setAnimation(g_ResourceManager.idleLeftCopy, this.frames, 10);
	    			this.animation = 1;
    			}
    			break;
    			
    		//Idle right
    		case 2:
    			if (this.animation != 2)
    			{
    				this.setAnimation(g_ResourceManager.idleRightCopy, this.frames, 10);
    				this.animation = 2;
    			}
    			break;
    			
    		//Idle up
    		case 3:
    			if (this.animation != 3)
    			{
    				this.setAnimation(g_ResourceManager.idleUpCopy, this.frames, 10);
    				this.animation = 3;
    			}
    			break;
    			
    		//Idle down
    		case 4:
    			if (this.animation != 4)
    			{
    				this.setAnimation(g_ResourceManager.idleDownCopy, this.frames, 10);
    				this.animation = 4;
    			}
    			break;
    			
    		//Move left
    		case 5:
    			if (this.animation != 5)
    			{
    				this.setAnimation(g_ResourceManager.moveLeftCopy, this.frames, 10);
    				this.animation = 5;
    			}
    			break;
    			
    		//Move right
    		case 6:
    			if (this.animation != 6)
    			{
    				this.setAnimation(g_ResourceManager.moveRightCopy, this.frames, 10);
    				this.animation = 6;
    			}
    			break;
    			
    		//Move up
    		case 7:
    			if (this.animation != 7)
    			{
    				this.setAnimation(g_ResourceManager.moveUpCopy, this.frames, 10);
    				this.animation = 7;
    			}
    			break;
    			
    		//Move down
    		case 8:
    			if (this.animation != 8)
    			{    				
    				this.setAnimation(g_ResourceManager.moveDownCopy, this.frames, 10);
    				this.animation = 8;
    			}
    			break;
    			
    		//Invalid animation setting
    		default:
    			break;
    	}
    }


    /*  Updates the object
        @param dt The time since the last frame in seconds
        @param context The drawing context
        @param xScroll The global scrolling value of the x axis
        @param yScroll The global scrolling value of the y axis  */
    this.update = function (/**Number*/ dt, /**CanvasRenderingContext2D*/context, /**Number*/ xScroll, /**Number*/ yScroll)
    {
    	if (!this.dead){
    		
    		if (this.ticks >= this.record.xCoords.length)
    		{
    			this.shutdown();
    		}
    		else
    		{
	    		this.x = this.record.xCoords[this.ticks];
	    		this.y = this.record.yCoords[this.ticks];
	    		this.updateAnimation(this.record.animations[this.ticks]);
	    		
		    	//While moving, check for collisions
				this.collisionCheck();
				
		    	if (this.hasItem)
		    	{
		    		g_Gem.x = this.x+8;
		    		g_Gem.y = this.y+10;
		    	}
		       	
		       	this.ticks++;
	        }
       	}
       	
    }
    
    //Collision check for Timetanium
    this.collisionCheck = function()
    {
    	
    	//Check collisions with timetanium
    	if (g_Gem.owner == null)
    	{
    		if (this.collisionArea().intersects(g_Gem.collisionArea()))
    		{
    			g_Gem.changeOwner(this);
    		}
    	}
    	//Check collisions with timetanium carrier
    	else if (g_Gem.owner != this)
    	{
    		if (this.collisionArea().intersects(g_Gem.owner.collisionArea()))
    		{
    			if (g_Gem.cooldown <= 0) {
	    			if (!this.touching)
	    			{
	    				g_Gem.changeOwner(this);
	    				g_Gem.cooldown = g_Gem.maxCooldown;
	    				this.touching = true;
	    			}
    			}
    		}
    		else this.touching = false;
    	} 
    	
    }
    
    this.shutdown = function()
    {
    	this.dead = true;
    	this.ticks = 0;
        /*this.x = -100;
        this.y = -100;*/
        this.shutdownAnimatedGameObject();
    }
    
    //Called when this entity picks up an item
    this.getItem = function()
    {
    	this.hasItem = true;
    	
    	//play the copy get item sound (start and end positions, can be found in SoundManager)
    	g_SoundManager.playSound(1.135, 1.640);
    }
    
    //Called when this entity loses its held item
    this.loseItem = function()
    {
    	this.hasItem = false;
    	this.touching = true;
    }
    
}

CopyPlayer.prototype = new AnimatedGameObject;