/*  
    A class to represent the player on the screen for the last level, which looks like a copy
    @Author Jonathan Saunders and Ethan White
    @Original <a href="mailto:matthewcasperson@gmail.com">Matthew Casperson</a>
    @class
*/
function FinalPlayer()
{
    
    this.hasItem = false;   //true if player has an item, false otherwise
    this.canGetItem = true;     //true if the player can get the item, false otherwise
    
    this.touching = false;   //true if the item was just lost, and can't be regained during this collision
    
    this.moving = false;    //True if the player is moving via the mouse or touchscreen, false otherwise
    
    //For keyboard input
    this.left = false;     //True if the player is moving left
    this.right = false;     //True if the player is moving right
    this.up = false;      //True if the player is moving up
    this.down = false;     //True if the player is moving down
    
    this.animation = 4;     // Keeps track of which animation is currently being played
    /*
     * Animations reference:
     * 1 = Idle left
     * 2 = Idle right
     * 3 = Idle up
     * 4 = Idle down
     * 5 = Run left
     * 6 = Run right
     * 7 = Run up
     * 8 = Run down
     * More can be added as necessary
     */
    
    this.frames = 8;        //Number of frames in player animations
    
    //The player's current destination
    this.destX = 0;
    this.destY = 0;
    
    //The difference between where the player clicks and moves
    this.diffX = 0;
    this.diffY = 0;
    
    //The player's destination during collisions
    this.tempDestX = 0;
    this.tempDestY = 0;
    
    //The player's position at the beginning of movement
    this.startX = 0;
    this.startY = 0;
    
    //The distance from the player's location to their destination
    this.dx = 0;
    this.dy = 0;
    this.dTotal = 0;
    
    this.level = null;		// A reference to the level object  @type Level
    
    //Used for the credits
    this.ticks = 0;
    
    //True if credits are rolling
    this.rolling = false;
    
    //The credits
    this.credits = new Text().startupText(0,0, "", "");
    
    // Initialises this object
    this.startupFinalPlayer = function(level)
    {
        this.startupAnimatedGameObject(g_ResourceManager.idleDownCopy, level.startX, level.startY, 6, this.frames, 10);
        this.level = level;
        this.destX = this.x;
        this.destY = this.y;
        this.xCoords = new Array();
        this.yCoords = new Array();
        this.animations = new Array();
        //Set where the player moves relative to click coordinates
        this.diffX = 15;
        if (g_GameObjectManager.isMobile) {
        	this.diffY = 60;
        }
        else this.diffY = 42;
        
        //Adjust the speed
        g_Speed = 350 * g_RatioX;
        
        //Adjust collision area
        this.collideY = 35 * g_RatioY;
        this.collideHeight = -40 * g_RatioY;
        this.collideX = 5 * g_RatioX;
        this.collideWidth = -10 * g_RatioX;
        return this;
    }

       
    // Called when a mousebutton is pressed  @param event Event Object
    this.onMouseDown = function(event)
    {
        this.moving = true;
    }
    
    //Same as above, but for touchscreens
    this.touchStart = function(event)
    {    	
    	this.moving = true;
    }
    

    // Called when a mousebutton is released  @param event Event Object
    this.onMouseUp = function(event)
    {
    	this.moving = false;
    }
    
    //Same as above, but for touchscreens
    this.touchEnd = function(event)
    {
    	
    	this.moving = false;
    }
   
   
    this.hitLevelEnd = function()
    {
    	//calls sound insertion
    	//window.g_SoundTrack.insertSound(9);
		this.hasItem = false;
		
		//plays success sound (start and end positions, can be found in SoundManager)
		g_SoundManager.playSound(2.008, 3.075);
		
		//If the player has enough copies, clear and go to next level
		if (g_Copies.length >= this.level.passNum)
		{
			this.level.nextLevel();
			g_ApplicationManager.emptyArray();
		}
		else 
		{
			g_Copies.push(new CopyPlayer());
			g_Records.push(new PastRecord().startupPastRecord(this.xCoords, this.yCoords, this.animations));
		}
        g_ApplicationManager.hop(2);    	
	
    }


    /*  Updates the current animation depending on the movement
        of the player.  */
    this.updateAnimation = function(index)
    {
    	
    	switch(index) {
    		
    		//Idle left
    		case 1:
    			if (this.animation != 1)
    			{
    				this.setAnimation(g_ResourceManager.idleLeftCopy, this.frames, 10);
    				this.animation = 1;
    			}
    			break;
    			
    		//Idle right
    		case 2:
    			if (this.animation != 2)
    			{
    				this.setAnimation(g_ResourceManager.idleRightCopy, this.frames, 10);
    				this.animation = 2;
    			}
    			break;
    			
    		//Idle up
    		case 3:
    			if (this.animation != 3)
    			{
    				this.setAnimation(g_ResourceManager.idleUpCopy, this.frames, 10);
    				this.animation = 3;
    			}
    			break;
    			
    		//Idle down
    		case 4:
    			if (this.animation != 4)
    			{
    				this.setAnimation(g_ResourceManager.idleDownCopy, this.frames, 10);
    				this.animation = 4;
    			}
    			break;
    			
    		//Move left
    		case 5:
    			if (this.animation != 5)
    			{
    				this.setAnimation(g_ResourceManager.moveLeftCopy, this.frames, 10);
    				this.animation = 5;
    			}
    			break;
    			
    		//Move right
    		case 6:
    			if (this.animation != 6)
    			{
    				this.setAnimation(g_ResourceManager.moveRightCopy, this.frames, 10);
    				this.animation = 6;
    			}
    			break;
    			
    		//Move up
    		case 7:
    			if (this.animation != 7)
    			{
    				this.setAnimation(g_ResourceManager.moveUpCopy, this.frames, 10);
    				this.animation = 7;
    			}
    			break;
    			
    		//Move down
    		case 8:
    			if (this.animation != 8)
    			{    				
    				this.setAnimation(g_ResourceManager.moveDownCopy, this.frames, 10);
    				this.animation = 8;
    			}
    			break;
    			
    		//Invalid animation setting
    		default:
    			break;
    	}
    }


    /*  Updates the object
        @param dt The time since the last frame in seconds
        @param context The drawing context
        @param xScroll The global scrolling value of the x axis
        @param yScroll The global scrolling value of the y axis  */
    this.update = function (/**Number*/ dt, /**CanvasRenderingContext2D*/context, /**Number*/ xScroll, /**Number*/ yScroll)
    {
    	
    	//Updates the player's destination
    	if (/*if the mousebutton is down/if the player is touching the screen*/(this.moving)
    			/*The rest of the 'if' statement ensures that the player doesn't try to move to where they're standing*/ 
    			&& ((Math.abs(this.x+this.diffX - g_MouseX) > 3) 
    				|| (Math.abs(this.y+this.diffY - g_MouseY) > 3))) {
    		
    		this.destX = g_MouseX - this.diffX;
    		this.destY = g_MouseY - this.diffY;
    		
    		//Used to determine trajectory of movement
    		this.dx = this.destX - this.x;
    		this.dy = this.destY - this.y;
    		this.dTotal = Math.abs(this.dx) + Math.abs(this.dy);
    		this.startX = this.x;
    		this.startY = this.y;
    		
			//Determine which animation to use
			if (this.dx > 0 && this.dx >= Math.abs(this.dy))
			{
				this.updateAnimation(6);
			}
			else if (Math.abs(this.dx) >= Math.abs(this.dy))
			{
				this.updateAnimation(5);
			}
			else if (this.dy > 0)
			{
				this.updateAnimation(8);
			}
			else
			{
				this.updateAnimation(7);
			}
    	}
    	//document.getElementById("debug").innerHTML = this.touching;
		
		//Moves the player
		if ((this.destX != this.x) || (this.destY != this.y))
		{
			
			this.x += this.dx * (g_Speed*dt/this.dTotal);
			this.y += this.dy * (g_Speed*dt/this.dTotal);
			
			//Stop when the destination is reached
			if (Math.abs(this.x - this.startX) > Math.abs(this.dx) || Math.abs(this.y - this.startY) > Math.abs(this.dy))
			{
				this.stop();
			}
			
			//While moving, check for collisions
			this.collisionCheck();

		}
		//Check for keyboard input, but only if no mouse input
		else if (this.left || this.right || this.up || this.down)
		{
			if (this.left && this.up)
			{
				this.updateAnimation(5);
				this.x -= (g_Speed*dt)/2;
				this.y -= (g_Speed*dt)/2;
			}
			else if (this.left && this.down)
			{
				this.updateAnimation(5);
				this.x -= (g_Speed*dt)/2;
				this.y += (g_Speed*dt)/2;
			}
			else if (this.right && this.up)
			{
				this.updateAnimation(6);
				this.x += (g_Speed*dt)/2;
				this.y -= (g_Speed*dt)/2;
			}
			else if (this.right && this.down)
			{
				this.updateAnimation(6);
				this.x += (g_Speed*dt)/2;
				this.y += (g_Speed*dt)/2;
			}
			else if (this.left)
			{
				this.updateAnimation(5);
				this.x -= g_Speed*dt;
			}
			else if (this.right)
			{
				this.updateAnimation(6);
				this.x += g_Speed*dt;
			}
			else if (this.up)
			{
				this.updateAnimation(7);
				this.y -= g_Speed*dt;
			}
			else if (this.down)
			{
				this.updateAnimation(8);
				this.y += g_Speed*dt;
			}
			
			//While moving, check for collisions
			this.collisionCheck();
			
			this.destX = this.x;
			this.destY = this.y;
		}
		else this.stop();
		
		//Check for collisions with gem or other carrier
		this.collideTimetanium();

        // keep the player bound to the level
        if (this.x > g_GameObjectManager.canvas.width - this.baseW/this.frames - 22)
        {
            this.x = g_GameObjectManager.canvas.width - this.baseW/this.frames - 22;
            if (Math.abs(this.destY - this.y) < 3) this.stop();
        }
        if (this.x < 22)
        {
       	    this.x = 22;
       	    if (Math.abs(this.destY - this.y) < 3) this.stop();
   	    }
       	    
       	if (this.y > g_GameObjectManager.canvas.height - this.baseH)
       	{
	    	this.y = g_GameObjectManager.canvas.height - this.baseH;
	    	if (Math.abs(this.destX - this.x) < 3) this.stop();
    	}
		if (this.y < 0)
		{
       	    this.y = 0;
       	    if (Math.abs(this.destX - this.x) < 3) this.stop();
   	    }
   	    
    	if (this.hasItem)
    	{
    		g_Gem.x = this.x+8;
    		g_Gem.y = this.y+10;
    	}
    	
    	this.ticks += dt;
    	if (this.rolling)
    	{
    		this.rollCredits();
    	}
   	    
    }
    
    //Makes the player stop moving
    this.stop = function()
    {
    	
		this.destX = this.x;
		this.destY = this.y;
		
		//Set idle animation
		switch(this.animation) {
			
			case 5:
				this.updateAnimation(1);
				break;
				
			case 6:
				this.updateAnimation(2);
				break;
				
			case 7:
				this.updateAnimation(3);
				break;
			
			case 8:
				this.updateAnimation(4);
				break;
		}
    }
    
    //Collision check for obstacles or Timetanium
    this.collisionCheck = function()
    {
    	//Check for collisions with obstacles
    	for (i = 0; i < g_Obstacles.length; i++)
    	{
    		if (this.collisionArea().intersects(g_Obstacles[i].collisionArea()))
    		{
    			var tempX = this.x + this.collideX;
    			var tempY = this.y + this.collideY;
    			var tempWidth = this.baseW/this.frames + this.collideWidth;
    			var tempHeight = this.baseH + this.collideHeight;
    			
    			//Calcualte the distance between each edge of the player and the opposite edge of the obstacle
    			var p_collide_left = Math.abs(tempX - (g_Obstacles[i].x + g_Obstacles[i].baseW));
    			var p_collide_right = Math.abs(tempX + tempWidth - g_Obstacles[i].x);
    			var p_collide_up = Math.abs(tempY - (g_Obstacles[i].y + g_Obstacles[i].baseH));
    			var p_collide_down = Math.abs(tempY + tempHeight - g_Obstacles[i].y);
    			
    			/* Whichever distance is the smallest, that's where the collision happened.
    			 * The player should move along that edge toward their destination.
    			 * Also, the player shouldn't overlap with the obstacle.
    			 */
    			switch (Math.min(p_collide_left, p_collide_right, p_collide_up, p_collide_down))
    			{
    				//Collision is on the player's left edge
    				case p_collide_left:
    					this.x = g_Obstacles[i].x + g_Obstacles[i].baseW + 1 - this.collideX;
    					break;
    					
    					
    				//Collision is on the player's right edge
    				case p_collide_right:
    					this.x = g_Obstacles[i].x - (this.collideX + this.baseW/this.frames) - 1 - this.collideWidth;
    					break;
    					
    				//Collision is on the player's top edge
    				case p_collide_up:
    					this.y = g_Obstacles[i].y + g_Obstacles[i].baseH + 1 - this.collideY;
    					break;
    					
    				//Collision is on the player's bottom edge
    				case p_collide_down:
    					this.y = g_Obstacles[i].y - (this.baseH + this.collideY) - 1 - this.collideHeight;
    					break;
    					
    				//Something went wrong
    				default: break;
    			}
    		}

    		
    	}
    	
	
    	if (this.moving)
    	{    	
	    	//Recalculate trajectory of movement
			this.dx = this.destX - this.x;
			this.dy = this.destY - this.y;
			this.dTotal = Math.abs(this.dx) + Math.abs(this.dy);
			this.startX = this.x;
			this.startY = this.y;
		}
    }
    
    //Check for collisions with timetanium or its carrier
    this.collideTimetanium = function() {
    	//Check collisions with timetanium
    	if (g_Gem.owner == null)
    	{
    		if (this.collisionArea().intersects(g_Gem.collisionArea()))
    		{
    			g_Gem.changeOwner(this);
    		}
    	}
    	//Check collisions with timetanium carrier
    	else if (g_Gem.owner != this)
    	{
    		if (this.collisionArea().intersects(g_Gem.owner.collisionArea()))
    		{
    			if (g_Gem.cooldown <= 0) {
	    			if (!this.touching)
	    			{
	    				g_Gem.changeOwner(this);
	    				g_Gem.cooldown = g_Gem.maxCooldown;
	    				this.touching = true;
	    			}
    			}
    		}
    		else this.touching = false;
    	} 
    }
    
    //Called when this entity picks up an item
    this.getItem = function()
    {	
    	if (this.canGetItem) {
	    	//Play a sound
	    	//if (this.hasItem = false) window.g_SoundTrack.setAdditive("swipe");
	    	this.hasItem = true;
	    	
	    	//play get item sound (start and end positions, can be found in SoundManager)
	    	g_SoundManager.playSound(0.122, 0.610);
	    	
	    	//Make any portals appear
	    	for (i = 0; i < g_Portals.length; i++)
	    	{
	    		g_Portals[i].appear();
	    	}
    	}
    }
    
    //Called when this entity loses its held item
    this.loseItem = function()
    {
    	//Play a sound
    	//if (this.hasItem = true) window.g_SoundTrack.setAdditive("swipe");
    	this.hasItem = false;
    	this.touching = true;
    	
    	//Make any portals disappear
    	for (i = 0; i < g_Portals.length; i++)
    	{
    		g_Portals[i].disappear();
    	}
    }
    
	this.shutdown = function()
	{
		this.shutdownAnimatedGameObject();
	}
	
	//Activates keyboard-based movement
	this.keyDown = function(event)
	{
		var key = (window.event) ? event.which : event.keyCode;
		switch(key)
		{
			//Left arrow or 'a' key
			case 37: case 65:
				this.right = false;
				this.left = true;
				break;
			
			//Right arrow or 'd' key	
			case 39: case 68:
				this.left = false;
				this.right = true;
				break;
				
			//Up arrow or 'w' key
			case 38: case 87:
				this.down = false;
				this.up = true;
				break;
				
			//Down arrow or 's' key
			case 40: case 83:
				this.up = false;
				this.down = true;
				break;
		}
	}
	
	//Ends keyboard-based movement
	this.keyUp = function(event)
	{
		var key = (window.event) ? event.which : event.keyCode;
		switch(key)
		{
			//Left arrow or 'a' key
			case 37: case 65:
				this.left = false;
				break;
			
			//Right arrow or 'd' key	
			case 39: case 68:
				this.right = false;
				break;
				
			//Up arrow or 'w' key
			case 38: case 87:
				this.up = false;
				break;
				
			//Down arrow or 's' key
			case 40: case 83:
				this.down = false;
				break;
		}
	}
	
	//Might as well put the credits here
	this.rollCredits = function()
	{
		/*
			this.credits.x = 180;
			this.credits.label = "Returning to title screen in: ";
			this.credits.data = 5;
			g_Clock = 5;
			window.setTimeout(function() {g_ApplicationManager.openMainMenu(1);}, 5000);
		 */
		if (this.ticks > 24.5)
		{
			this.credits.x = 180;
			this.credits.label = "Returning to title screen in: ";
			this.credits.data = 5;
			g_Clock = 5;
			window.setTimeout(function() {g_ApplicationManager.openMainMenu(1);}, 5000);
		}
		if (this.ticks > 22)
		{
			this.credits.x = 100;
			this.credits.label = "Special Thanks: ";
			this.credits.data = "Brian Moriarty, Anthony Russo";
		}
		else if (this.ticks > 18.5)
		{
			this.credits.x = 165;
			this.credits.label = "Advised by: ";
			this.credits.data = "Joseph Farbrook";
		}
		else if (this.ticks > 15.5)
		{
			this.credits.x = 180;
			this.credits.label = "Lead Tech: ";
			this.credits.data = "Ethan White";
		}
		else if (this.ticks > 11)
		{
			this.credits.x = 50;
			this.credits.label = "Backup Art; Sound and Mobile Tech: ";
			this.credits.data = "Jonathan Saunders";
		}
		else if (this.ticks > 8)
		{
			this.credits.x = 175;
			this.credits.label = "Lead Art: ";
			this.credits.data = "Edward Golden";
		}
		else if (this.ticks > 6.5)
		{
			this.credits.x = 175;
			this.credits.y = 30;
			this.credits.label = "Thank you for playing!";
			//this.credits.data = "Edward Golden";
		}
	}
	    
}

FinalPlayer.prototype = new AnimatedGameObject;