/**
    A manager for all the objects in the game
    @Author Jonathan Saunders
    @Original <a href="mailto:matthewcasperson@gmail.com">Matthew Casperson</a>
    @class
*/
function GameObjectManager()
{
    // An array of game objects  @type Arary
    this.gameObjects = new Array();
    
    // An array of new game objects  @type Arary
    this.addedGameObjects = new Array();
    
    // An array of removed game objects  @type Arary
    this.removedGameObjects = new Array();
    
    // The time that the last frame was rendered   @type Date
    this.lastFrame = new Date().getTime();
    
    // The global scrolling value of the x axis  
    this.xScroll = 0;
    
    // The global scrolling value of the y axis  
    this.yScroll = 0;
    
    // A reference to the canvas element   @type HTMLCanvasElement
    this.canvas = null;
    
    // A reference to the 2D context of the canvas element  @type CanvasRenderingContext2D
    this.context2D = null;
    
    // A reference to the in-memory canvas used as a back buffer  @type HTMLCanvasElement
    this.backBuffer = null;
    
    // A reference to the backbuffer 2D context  @type CanvasRenderingContext2D
    this.backBufferContext2D = null;
    
    // True if the canvas element is supported, false otherwise
    this.canvasSupported = false;
    
	// True if the resources supplied to the ResourceManager are all loaded, false otherwise
    this.resourcesLoaded = false;
    
    //Array that holds each touch
	this.touches = [];
    
	// The current colour of the loading screen
	this.loadingScreenCol = 0;
	
	/** The direction of the changes to the loading screen colour.
		1 = colour moving towards white
		-1 = colour moving topwards black
        @type Number */	
	this.loadingScreenColDirection = 1;
	
	// How quickly to change the loading screen colour per second
	this.loadingScreenColSpeed = 255;
	
	//keeps track of the amount of time passed
	this.time = 0;
	
	//whether or not the device is mobile
	this.isMobile = false;
	

    /** Initialises this object
        @return A reference to the initialised object */
    this.startupGameObjectManager = function()
    {
        // set the global pointer to reference this object
        g_GameObjectManager = this;
		
		//document.getElementById("debug").innerHTML = 0;
		
        // get references to the canvas elements and their 2D contexts
        this.canvas = document.getElementById('canvas');
        
        if( window.addEventListener ){
        	
	        
			//Registering mouse events
			this.canvas.addEventListener("mousedown", onMouseDown, false);
			this.canvas.addEventListener("mouseup", onMouseUp, false);
			this.canvas.addEventListener("mousemove", onMouseMove, false);
	        
	        //Registering touch events
	        this.canvas.addEventListener("touchstart", touchStart, false);
	        this.canvas.addEventListener("touchmove", touchMove, false);
	        this.canvas.addEventListener("touchend", touchEnd, false);
        }
        
        //Checks if the device is a mobile device
		if (Boolean(navigator.userAgent.match(/(iPhone|iPod|iPad|BlackBerry|PlayBook|Android|Windows Phone OS|Windows CE|Windows Mobile|Mobile|GT-P1000|SGH-T849|SHW-M180S|PalmOS|PalmSource| Pre\/|Opera Mini|IEMobile|SonyEricsson|smartphone|Kindle)/))) 
			{this.isMobile = true;}
               
         
        //Resizes the canvas if on mobile
        if (this.isMobile){
        	this.canvas.style.position = "fixed";
        	
		    if (window.orientation == 0) {
				g_RatioX = ((window.innerHeight+(screen.height - window.innerHeight)) / 600);
				g_RatioY = ((window.innerWidth+12-(screen.height - window.innerHeight)) / 360);
				
	        	this.canvas.setAttribute("width", window.innerHeight+(screen.height - window.innerHeight));
	        	this.canvas.setAttribute("height", window.innerWidth+12-(screen.height - window.innerHeight));
		    }
		    if ((window.orientation == 90) || (window.orientation == -90)) {
	        	g_RatioX = (window.innerWidth / 600);
	        	g_RatioY = (window.innerHeight / 360);
	        	
	        	this.canvas.setAttribute("width", window.innerWidth);
	        	this.canvas.setAttribute("height", window.innerHeight);
		    }
        }
        
        // if the this.canvas.getContext function does not exist it is a safe bet that
        // the current browser does not support the canvas element.
        // in this case we don't go any further, which will save some debuggers (like
        // the IE8 debugger) from throwing up a lot of errors.
        if (this.canvas.getContext)
        {
            this.canvasSupported = true;
            this.context2D = this.canvas.getContext('2d');
            this.backBuffer = document.createElement('canvas');
            this.backBuffer.width = this.canvas.width;
            this.backBuffer.height = this.canvas.height;
            this.backBufferContext2D = this.backBuffer.getContext('2d');
        } 


        // create a new ResourceManager
        new ResourceManager().startupResourceManager(
            [{name: 'moveUp', src: 'Visual/Character_Animations/DanGuymore_Walk_Up.png'},
            {name: 'moveDown', src: 'Visual/Character_Animations/DanGuymore_Walk_Down.png'},
            {name: 'moveLeft', src: 'Visual/Character_Animations/DanGuymore_Walk_Left.png'},
            {name: 'moveRight', src: 'Visual/Character_Animations/DanGuymore_Walk_Right.png'},
            {name: 'idleUp', src: 'Visual/Character_Animations/DanGuymore_Idle_Up.png'},
            {name: 'idleDown', src: 'Visual/Character_Animations/DanGuymore_Idle_Down.png'},
            {name: 'idleLeft', src: 'Visual/Character_Animations/DanGuymore_Idle_Left.png'},
            {name: 'idleRight', src: 'Visual/Character_Animations/DanGuymore_Idle_Right.png'},
            
            {name: 'moveUpCopy', src: 'Visual/Character_Animations/Copy_Walk_Up.png'},
            {name: 'moveDownCopy', src: 'Visual/Character_Animations/Copy_Walk_Down.png'},
            {name: 'moveLeftCopy', src: 'Visual/Character_Animations/Copy_Walk_Left.png'},
            {name: 'moveRightCopy', src: 'Visual/Character_Animations/Copy_Walk_Right.png'},
            {name: 'idleUpCopy', src: 'Visual/Character_Animations/Copy_Idle_Up.png'},
            {name: 'idleDownCopy', src: 'Visual/Character_Animations/Copy_Idle_Down.png'},
            {name: 'idleLeftCopy', src: 'Visual/Character_Animations/Copy_Idle_Left.png'},
            {name: 'idleRightCopy', src: 'Visual/Character_Animations/Copy_Idle_Right.png'},
            
            {name: 'newGameButton', src: 'Visual/NewGame_Button.png'},
            {name: 'instructionsButton', src: 'Visual/Instructions_Button.png'},
            {name: 'timetanium', src: 'Visual/timetanium.png'},
            {name: 'mainmenu', src: 'Visual/Title.png'},
            {name: 'instructions', src: 'Visual/instructions.png'},
            {name:'countdown', src: 'Visual/Countdown.png'},
            {name: 'portal', src: 'Visual/time_portal.png'},
            {name: 'greyPortal', src: 'Visual/grey_time_portal.png'},
            {name: 'textGlow', src: 'Visual/glow.png'},
            {name: 'winHop', src: 'Visual/win_hop.png'},
            {name: 'loseHop', src: 'Visual/lose_hop.png'},
            {name: 'winPortalOut', src: 'Visual/win_portal_anim.png'},
            {name: 'losePortalOut', src: 'Visual/lose_portal_anim.png'},
            {name: 'portIn', src: 'Visual/port_in.png'},
            
            {name: 'presentEra', src: 'Visual/era1.png'},
            {name: 'sixtiesEra', src: 'Visual/era2.png'},
            {name: 'medievalEra', src: 'Visual/era3.png'},
            {name: 'prehistoricEra', src: 'Visual/era4.png'},
            
            {name: 'stage', src: 'Visual/Present_Era/Stage.png'},
            {name: 'chemistry', src: 'Visual/Present_Era/chemistry.png'},
            {name: 'waste', src: 'Visual/Present_Era/rad_waste.png'},
            {name: 'chair', src: 'Visual/Present_Era/chair.png'},
            {name: 'desk', src: 'Visual/Present_Era/desk.png'},
            {name: 'tipped_chair', src: 'Visual/Present_Era/tipped_chair.png'},
            
            {name: 'stage60s', src: 'Visual/60s_era/Stage.png'},
            {name: 'aquarium', src: 'Visual/60s_era/aquarium.png'},
            {name: 'display', src: 'Visual/60s_era/display_cabinet.png'},
            {name: 'atomic_clock', src: 'Visual/60s_era/atomic_clock.png'},
            {name: 'roll_chair', src: 'Visual/60s_era/roll_chair.png'},
            {name: 'cooler', src: 'Visual/60s_era/water_cooler.png'},
            {name: 'chemicals', src: 'Visual/60s_era/chemicals.png'},
            {name: 'teak_desk', src: 'Visual/60s_era/teak_desk.png'},
            
            {name: 'stageMedieval', src: 'Visual/Medieval/Stage.png'},
            {name: 'alchemy', src: 'Visual/Medieval/Alchemy_Table.png'},
            {name: 'barrel', src: 'Visual/Medieval/barrel.png'},
            {name: 'lectern', src: 'Visual/Medieval/lectern.png'},
            {name: 'books', src: 'Visual/Medieval/book_stack.png'},
            {name: 'bookshelf', src: 'Visual/Medieval/Bookshelf.png'},
            {name: 'cauldron', src: 'Visual/Medieval/cauldron.png'},
            {name: 'chest', src: 'Visual/Medieval/chest.png'},
            {name: 'crystal_ball', src: 'Visual/Medieval/crystal_ball.png'},
            {name: 'stool', src: 'Visual/Medieval/work_stool.png'},
            

			{name: 'stagePrehistoric', src: 'Visual/Prehistoric/stage.png'},
			{name: 'dino_skull', src: 'Visual/Prehistoric/dino_skull.png'},
			{name: 'fire', src: 'Visual/Prehistoric/fire.png'}]);
            
        new SoundManager().startupSoundManager();
        

        // use setInterval to call the draw function
        setInterval(function(){g_GameObjectManager.draw();}, SECONDS_BETWEEN_FRAMES);

        return this;
    }
    
    // The render loop
    this.draw = function ()
    {
        // calculate the time since the last frame
        var thisFrame = new Date().getTime();
        var dt = (thisFrame - this.lastFrame)/1000;
        this.lastFrame = thisFrame;
        
        window.scrollTo(0, 1);
        
        if (!this.resourcesLoaded)
        {
            var numLoaded = 0;
            for (i = 0; i < g_ResourceManager.imageProperties.length; i++)
            {
                if (g_ResourceManager[g_ResourceManager.imageProperties[i]].complete)
                {
                    ++numLoaded;
                }
            }
            /*
            for (i = 0; i < g_SoundManager.soundProperties.length; ++i)
            {
                if ((g_SoundManager[g_SoundManager.soundProperties[i]].readyState == 4) || (g_SoundManager[g_SoundManager.soundProperties[i]].readyState == 3))
                {
                	++numLoaded;
                }
                else {
                	g_SoundManager[g_SoundManager.soundProperties[i]].load();
                	g_SoundManager[g_SoundManager.soundProperties[i]].volume = 0.0000001;
                	g_SoundManager[g_SoundManager.soundProperties[i]].play();
                	     		
                }
            }
			document.getElementById("debug").innerHTML = numLoaded + "/" + (g_ResourceManager.imageProperties.length + g_SoundManager.soundProperties.length) + " . " + dt;
*/
            if ( numLoaded == (g_ResourceManager.imageProperties.length /*+ g_SoundManager.soundProperties.length*/ ) )
            {
                // create a new ApplicationManager
                new ApplicationManager().startupApplicationManager(this.canvas.width, this.canvas.height);
                this.resourcesLoaded = true;
            }
            else
            {
                this.loadingScreenCol += this.loadingScreenColDirection * this.loadingScreenColSpeed * dt;
                if (this.loadingScreenCol > 255)
                {
                    this.loadingScreenCol = 255;
                    this.loadingScreenColDirection = -1;
                }
                else if (this.loadingScreenCol < 0)
                {
                    this.loadingScreenCol = 0;
                    this.loadingScreenColDirection = 1;
                }
                this.context2D.fillStyle = "rgb(" + parseInt(this.loadingScreenCol) + "," + parseInt(this.loadingScreenCol) + "," + parseInt(this.loadingScreenCol) + ")";
                this.context2D.fillRect (0, 0, this.canvas.width, this.canvas.height);
            }
        }
        
        
        // clear the drawing contexts
        if (this.canvasSupported && this.resourcesLoaded)
        {
        	
            this.backBufferContext2D.clearRect(0, 0, this.backBuffer.width, this.backBuffer.height);    
        
	        //Only do the following if the game is not paused
	        if (!g_Pause)
	        {
	            // first update all the game objects
	            for (var x = 0; x < this.gameObjects.length; x++)
	            {
	                if (this.gameObjects[x].update)
	                {
	                    this.gameObjects[x].update(dt, this.backBufferContext2D, this.xScroll, this.yScroll);
	                }
	            }
	            g_SoundManager.update(dt);
	            
	            this.addNewGameObjects();
            	this.removeOldGameObjects();
            	
	            // then draw the game objects
	            for (var x = 0; x < this.gameObjects.length; x++)
	            {
	                if (this.gameObjects[x].draw)
	                {
	                    this.gameObjects[x].draw(dt, this.backBufferContext2D, this.xScroll, this.yScroll);
	                }
	            }
	            
	            //document.getElementById("debug").innerHTML = g_DrawObjs - g_DrawnObjs;
	            
				//Update time
				this.updateTime(dt);
            	
	            // copy the back buffer to the displayed canvas
	            this.context2D.drawImage(this.backBuffer, 0, 0);
	           
           }
        }
    };
    
    this.playAnim = function(a)
    {
	    var anim = new AnimatedGameObject().startupAnimatedGameObject(a, ((this.canvas.width)/2)-(a.width/6/2), ((this.canvas.height)/2)-(a.height/2), 20, 6, 30);
    };
   
    this.updateTime = function(dt)
    {
    	g_Time = Math.round(this.time += dt);
    	//return Math.round(this.time += dt);
    };

    this.shutdownAll = function()
    {
        for (var x = 0; x < this.gameObjects.length; ++x)
        {
            if (this.gameObjects[x].shutdown)
            {
                this.gameObjects[x].shutdown();
            }
        }

        this.removeOldGameObjects();
    }
    
    /** Adds a new GameObject to the gameObjects collection
        @param gameObject The object to add */
    this.addGameObject = function(gameObject)
    {
        this.addedGameObjects.push(gameObject);
    };

    this.addNewGameObjects = function()
    {
        if (this.addedGameObjects.length != 0)
        {
            for (var x = 0; x < this.addedGameObjects.length; ++x)
            {
                this.gameObjects.push(this.addedGameObjects[x]);
            }

            this.addedGameObjects.clear();
            this.gameObjects.sort(function(a,b){return a.zOrder - b.zOrder;});
        }
    }

    /** Removes a GameObject from the gameObjects collection
        @param gameObject The object to remove */
    this.removeGameObject = function(gameObject)
    {
        this.removedGameObjects.push(gameObject);
    }

    this.removeOldGameObjects = function()
    {
        if (this.removedGameObjects.length != 0)
        {
            for (var x = 0; x < this.removedGameObjects.length; ++x)
            {
            	var index = this.gameObjects.indexOf(this.removedGameObjects[x]);
            	if (index != -1)
            	{
                	this.gameObjects.splice(index, 1);
                }
                delete this.removedGameObjects[x];
            }
            this.removedGameObjects.clear();
        }
    }

	if (!this.isMobile) {
		//Keyboard input handlers
	    document.onkeydown = function(event)
	    {
	    	//Pause function
	        if (event.keyCode == 32)
	        {
	        	this.pause();
	        }
	        else{
		        g_player.keyDown(event);
		   }
	    }
	
	    document.onkeyup = function(event)
	    {
	        g_player.keyUp(event);
	    }
	    
	    this.pause = function()
	    {
	    	if (g_Pause) g_Pause = false;
	    	else g_Pause = true;
	    }
   }
}



function load(event)
	{
		document.getElementById("debug").innerHTML = 0;
		window.scrollTo(0,1);
		//setTimeout(window.scrollTo(x,y),100);
	}


function onMouseDown(event)
    {
        for (var x = 0; x < g_GameObjectManager.gameObjects.length; ++x)
        {
            if (g_GameObjectManager.gameObjects[x].onMouseDown)
            {
                g_GameObjectManager.gameObjects[x].onMouseDown(event);
            }
        }
    }
    
function onMouseUp(event)
    {
        for (var x = 0; x < g_GameObjectManager.gameObjects.length; ++x)
        {
            if (g_GameObjectManager.gameObjects[x].onMouseUp)
            {
                g_GameObjectManager.gameObjects[x].onMouseUp(event);
            }
        }
    }

function onMouseMove(event)
    {
		var totalOffsetX = 0;
	    var totalOffsetY = 0;
	    var currentElement = this;
	
	    do{
	        totalOffsetX += currentElement.offsetLeft;
	        totalOffsetY += currentElement.offsetTop;
	    }
	    while(currentElement = currentElement.offsetParent)
	
	    g_MouseX = event.pageX - totalOffsetX;
	    g_MouseY = event.pageY - totalOffsetY;
    }

function touchStart(event)
    {
    	//Store the coordinates of the touch
    	g_MouseX = event.touches[0].pageX;
    	g_MouseY = event.touches[0].pageY;
    	
    	//Process the event for any objects that use it
        for (var x = 0; x < g_GameObjectManager.gameObjects.length; ++x)
        {
            if (g_GameObjectManager.gameObjects[x].touchStart)
            {
                g_GameObjectManager.gameObjects[x].touchStart(event);
            }
        }
    }
    
function touchEnd(event)
    {
        for (var x = 0; x < g_GameObjectManager.gameObjects.length; ++x)
        {
            if (g_GameObjectManager.gameObjects[x].touchEnd)
            {
                g_GameObjectManager.gameObjects[x].touchEnd(event);
            }
        }
    }
    
function touchMove(event)
    {
    	//Prevents the touch event from being propagated to the browser
    	event.preventDefault();
    	
    	// get canvas position
	    var obj = this.canvas;
	    var top = 0;
	    var left = 0;
	    while (obj && obj.tagName != 'BODY') {
	        top += obj.offsetTop;
	        left += obj.offsetLeft;
	        obj = obj.offsetParent;
	    }

        g_MouseX = event.changedTouches[0].pageX - left + window.pageXOffset;
        g_MouseY = event.changedTouches[0].pageY - top + window.pageYOffset;
    }
