/**
    An object that causes the level to end when it it touched
    @Author Jonathan Saunders
    @Original <a href="mailto:matthewcasperson@gmail.com">Matthew Casperson</a>
    @class
*/
function LevelEndPost()
{
	this.iterations = 0;
	
	this.moving = false;     //true if this is a moving portal
	
	this.path = null;     //A list of the points this portal patrols (if moving portal)
	
	this.destID = 0;     //The index of the coordinate in this.path which this is moving toward
	
	this.speed = 100;     //The speed at which the portal moves, if it moves
	
	this.used = 0; 	//whether or not the portal has already been activated
	
	this.animated = false;	//Whether or not the animation has completed
	
	//The distance to the destination
    this.dx = 0;
    this.dy = 0;
    this.dTotal = 0;
    
    
    this.startX = 0;
    this.startY = 0;
    this.xSpeed = 0;
    this.ySpeed = 0;
    this.angle = 0;
    this.modX = 1;
    this.modY = 1;
	
    this.startupLevelEndPost = function(/**Image*/ image, /**Number*/ x, /**Number*/ y, /**Number*/ z)
    {
        this.startupAnimatedGameObject(image, x, y, z, 8, 10);
        this.path = new Array();
        this.path.push(new Coordinate().startupCoordinate(x, y));
        if (g_player != null) {
        	if (g_player.hasItem)
        	{
        		this.appear();
        	}
        }
        
	    this.startX = x;
	    this.startY = y;
	    
        return this;
    }
    
    //Called when the player picks up the timetanium
    this.appear = function()
    {
    	this.setAnimation(g_ResourceManager.portal, 8, 10);
    }
    
    //Called when the player loses the timetanium
    this.disappear = function()
    {
    	this.setAnimation(g_ResourceManager.greyPortal, 8, 10);
    }

    this.shutdown = function()
    {
        this.shutdownLevelEndPost();
    }

    this.shutdownLevelEndPost = function()
    {
        this.shutdownAnimatedGameObject();
    }

    /** Updates the object
        @param dt The time since the last frame in seconds
        @param context The drawing context
        @param xScroll The global scrolling value of the x axis
        @param yScroll The global scrolling value of the y axis  */
	this.update = function (/**Number*/ dt, /**CanvasRenderingContext2D*/context, /**Number*/ xScroll, /**Number*/ yScroll)
    {
    	//Move if this is a mover
    	if (this.moving)
    	{
	    	/*
	    	//if the portal is close to its destination, change directions
	    	//if ((this.path[this.destID].x/this.x < 1.1) || (this.path[this.destID].y/this.y < 1.1))
	    	if (Math.abs(this.x - this.path[this.destID].x) < 2 && Math.abs(this.y - this.path[this.destID].y) < 2)
	    	{
	    		if (this.destID >= this.path.length-1)
	    		{
	    			this.destID = 0;
	    		}
	    		else  {this.destID++;}
	    		//document.getElementById("debug").innerHTML = this.destID + ", " + this.modX + ", " + this.modY;
	    		
	    		if (this.x > this.path[this.destID].x) {this.modX = -1;}
	    		else {this.modX = 1;}

				if (this.y > this.path[this.destID].y) {this.modY = -1;}
	    		else {this.modY = 1;}

	    		//document.getElementById("debug").innerHTML = this.modX + ", " + this.xSpeed + ", " + this.angle + ", " + (this.x > this.path[this.destID].x);
	    	}
		    	this.angle = Math.atan(Math.abs(this.path[this.destID].x-this.path[this.destID-1].x)/Math.abs(this.path[this.destID].y-this.path[this.destID-1].y));
		    	this.xSpeed = Math.sin(this.angle) * this.modX;
		    	this.ySpeed = Math.cos(this.angle) * this.modY;
	    	document.getElementById("debug").innerHTML = this.angle * 180 / Math.PI;
	    	this.x += this.xSpeed;
	    	this.y += this.ySpeed;
	    	*/
    	
	    	//var test = new Text().startupText(300, 100, "Test: ", this.path[0].x + ", " + this.path[1].x + ", " + xSpeed);
	    	
	    	
    		//if the destination has been reached
	    	//if ((this.x == this.path[this.destID].x) && (this.y == this.path[this.destID].y))
	    	if (Math.abs(this.x - this.path[this.destID].x) < 2 && Math.abs(this.y - this.path[this.destID].y) < 2)
	    	{
	    		//if at the last destination, return to the first one, otherwise go to the next
	    		if (this.destID >= this.path.length-1)
	    		{
	    			this.destID = 0;
	    		}
	    		else {this.destID++;}
	    		
	    		//calculate new trajectory of movement
	    		this.dx = this.path[this.destID].x - this.x;
	    		this.dy = this.path[this.destID].y - this.y;
	    		this.dTotal = Math.abs(this.dx) + Math.abs(this.dy);
	    		this.startX = this.x;
	    		this.startY = this.y;
	    	}
	    	
	    	//this.x += this.dx * (this.speed*dt/this.dTotal);
			//this.y += this.dy * (this.speed*dt/this.dTotal);
	    	
	    	//if the destination has not been reached
	    	//if (!((this.x == this.path[this.destID].x) && (this.y == this.path[this.destID].y)))
	    	//{
	    		this.x += this.dx * (this.speed*dt/this.dTotal);
				this.y += this.dy * (this.speed*dt/this.dTotal);
				
				//if close enough to destination, just go there
				if (Math.abs(this.x - this.startX) > Math.abs(this.dx) || Math.abs(this.y - this.startY) > Math.abs(this.dy))
					{
						this.x = this.path[this.destID].x;
						this.y = this.path[this.destID].y;
					}
	    	//}
	    	
    	}
    	
        if (this.collisionArea().intersects(g_player.collisionArea()))
        {
	    	/* Check if the player has the item. If so, success. If not, backtrack.*/
	    	if (g_player.hasItem && this.used == 0)
	    	{
	    		this.used = 1;
	    		
	    		g_GameObjectManager.playAnim(g_ResourceManager.winPortalOut);
	    		
	    		g_Gem.shutdown();
	        	window.setTimeout('g_player.hitLevelEnd()', 200);
	        	
	        	if ((this.timer <= .2) && (!this.animated))
				{
					g_player.hitLevelEnd();
					this.animated = true;
				}
        	}
        	
        	
        }
        
        for (i=0; i < g_Copies.length; i++)
        {
        	if (this.collisionArea().intersects(g_Copies[i].collisionArea()) && g_Copies[i].hasItem && this.used == 0)
	        {
	        	this.used = 1;
	    		g_GameObjectManager.playAnim(g_ResourceManager.losePortalOut);
	        	
	    		g_Gem.shutdown();
	        	window.setTimeout('g_Copies[0].hitLevelEnd()', 200);
	        }
		}
    }
    
}
LevelEndPost.prototype = new AnimatedGameObject;