/**
    Represents the Timetanium in the game
    @Author Jonathan Saunders
    @Original <a href="mailto:matthewcasperson@gmail.com">Matthew Casperson</a>
    @class
*/
function Powerup()
{
    
    this.value = 0;  // The value of the powerup

	this.owner = null; //A reference to the game object currently carrying this powerup
	
	this.cooldown = 0;  //The number of loops one must wait before stealing the gem if it was just stolen 
	
	this.maxCooldown = 50;  //The amount of time the cooldown is set to, when activated
	

    /** Initialises this object
        @param value        The value (score) of this powerup
        @param image        The image to be displayed
        @param x            The position on the X axis
        @param y            The position on the Y axis
        @param z            The depth
        @param frameCount   The number of animation frames in the image
        @param fps          The frames per second to animate this object at  */
    this.startupPowerup = function(/**Number*/ value, /**Image*/ image, /**Number*/ x, /**Number*/ y, /**Number*/ z, /**Number*/ frameCount, /**Number*/ fps)
    {
        this.startupAnimatedGameObject(image, x, y-10, z, frameCount, fps);
        this.value = value;
        g_Gem = this;
        return this;
    }

    this.shutdownPowerup = function()
    {
        this.shutdownAnimatedGameObject();
    }

    this.shutdown = function()
    {
        this.shutdownPowerup();
    }

    /** Updates the object
        @param dt The time since the last frame in seconds
        @param context The drawing context
        @param xScroll The global scrolling value of the x axis
        @param yScroll The global scrolling value of the y axis  */
	this.update = function (/**Number*/ dt, /**CanvasRenderingContext2D*/context, /**Number*/ xScroll, /**Number*/ yScroll)
    {
        //this.collidePlayer();
        //this.collideCopy();
        if (this.cooldown > 0) {
        	this.cooldown--;
        }
        
    }
    
    /* checks for collisions against the player
    this.collidePlayer = function()
    {
    	if (this.collisionArea().intersects(g_player.collisionArea()))
        {
        	if (this.owner != g_player) this.changeOwner(g_player);
        	//g_player.hitPowerup();
        	this.x = g_player.x+8;
        	this.y = g_player.y+10;
            //this.shutdown();
            //g_score += this.value;
            //g_ApplicationManager.updateScore();
        }
    }
    
    // checks for collisions against every copy
    this.collideCopy = function()
    {
    	for (i=0; i < g_Copies.length; i++)
        {
	        if (this.collisionArea().intersects(g_Copies[i].collisionArea()))
	        {
	        	if (this.owner != g_Copies[i]) this.changeOwner(g_Copies[i]);
	        	this.x = g_Copies[i].x+8;
	        	this.y = g_Copies[i].y+10;
	        }
        }
    }*/
    
    //Kills previous owner, acknowledges new one
    this.changeOwner = function(/*Player or copy*/player)
    {
    	if (this.owner != null) this.owner.loseItem();
    	this.owner = player;
    	player.getItem();
    }
    
}

Powerup.prototype = new AnimatedGameObject;